#!/usr/bin/perl
## ######### PROJECT NAME : ##########
## 
## LibNetwork.pm for LibNetwork
## 
## ######### PROJECT DESCRIPTION : ###
## 
## This library is based on Mickael Sardinha's network lib.
## This is just a assembly of the different files in addition
## to a lot of hacks.
## 
## ###################################
## 
## Made by Boutonnet Alexandre
## Login   <aboutonnet@intellique.com>
## 
## Started on  Thu Mar  6 13:03:15 2008 Boutonnet Alexandre
## Last update Mon Jun  8 14:21:57 2009 Boutonnet Alexandre
## 
## ###################################
##

package INTLIQ::LibNetwork;

use strict;
use warnings;

# For debug
use Data::Dumper;

use Net::IP qw(ip_is_ipv4);

# Definitions of statics files
my $config_file_path	= "/etc/network/interfaces";

# Definitions of used binaries
my $cmd_ifconfig	= _test_cmd("ifconfig", "/sbin/ifconfig");
my $cmd_ethtool		= _test_cmd("ethtool", "/usr/sbin/ethtool");
my $cmd_pump		= _test_cmd("pump", "/sbin/pump");

# Definitions of errors strings
my $not_open_file	= "The file $config_file_path does not exist or can't be opened.\n";
my $error_ifconfig	= "There was an error during the execution of the command: $cmd_ifconfig.\n";

my $param = {
    'address'	=> qr/\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/,
    'netmask'	=> qr/\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/,
    'broadcast'	=> qr/\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/,
    'network'	=> qr/\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/,
    'gateway'	=> qr/\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/,
};

# Fonction de recuperation des configurations.
# Cette methode ne prend rien en parametre..
sub get_ifaces_info
{
    my ($interface) = @_;

    my $hash = {};

    if (!defined($interface))
    {
	_get_ifaces_on_boot($hash);
	_get_ifconf($hash);
    }
    else
    {
	_get_ifconf($hash, $interface);
    }
    _got_link($hash);

    return (0, $hash);
}

# Fonction d'application des configuration aux interfaces
# Cette methode prend en paramtre une hash ..
sub set_ifaces
{
    my ($hash) = @_;
    
    foreach my $iface (keys %{$hash})
    {
	if (exists($hash->{$iface}->{'on_boot'}))
	{
	    my ($error, $msg) = _do_interfaces_changes($iface, $hash->{$iface});
	    return ($error, $msg) if ($error);
	}
	if (exists($hash->{$iface}->{'ifconfig'}))
	{
	    my ($error, $msg) = active_iface($iface, $hash->{$iface}->{'ifconfig'});
	    return ($error, $msg) if ($error);
	}
    }

    return (0, '');
}

# Fonction d'application d'une configuration reseau sur une interface
# a travers la commande ifconfig : c'est a dire que cette configuration
# est volatile
# Cette fonction prend en parametre le nom de l'interface et la hash contenant
# la configuration.
sub active_iface
{
    my ($iface, $info) = @_;
    my $act_info = {};
    my $field;
    my @tab;

    _get_ifconf($act_info);

    #On stack les clefs dans un tableau ...
    foreach my $k (keys %{$info})
    {
	push @tab, $k;
    }
    # ... pour tester s'il y a des infos (idee de mickael :)
    return (1, "No data found") if ( ! scalar(@tab));
    
    $info->{'ip'} = $act_info->{$iface}->{'ifconfig'}{'ip'} if (!defined $info->{'ip'});
    $info->{'mask'} = $act_info->{$iface}->{'ifconfig'}{'mask'} if (!defined $info->{'mask'});

    my ($err, @errno_tab) = check_ip($info);
    if($err)
    {
	$field = join(', ', @errno_tab);
	return (1, "$field is/are not valid for $iface in active configuration");
    }
    
    if (defined $info->{'mode'} && $info->{'mode'} eq "dhcp" )
    {
	eval { qx!$cmd_pump -i $iface!;};
	return (1, "Error during activation of $iface in DHCP mode. Operations aborted") if ($?);	

	return (0, "Interface $iface configuration has been modified");
    }
    elsif ( ! defined $info->{'ip'} ||  ! defined $info->{'mask'})
    {
	return (1, "Error: Ip and Mask are required");
    }

    eval { qx!$cmd_ifconfig $iface inet $info->{'ip'} netmask $info->{'mask'} up !; };
    return (1, "Error during activation of $iface") if ($?);
    
    return (0, "Interface $iface configuration has been modified");
}

# Fonction de verification des adresses ip
# Cette fonction prend un hash en parametre
sub check_ip
{
    my $hash = shift;
    my @errno_var;

    foreach my $var (keys %{$hash} )
    {
       	if ($var =~ /(^ga|^ip|^mask|^bcast)/)
	{
	    if (!$hash->{$var} or (! ip_is_ipv4($hash->{$var})))
	    {
		push(@errno_var, $var);
	    }
	}
    }

    if (defined $errno_var[0])
    {
	return (1, @errno_var);
    }
    else
    {
	return (0, '');
    }
}

# Fonction d'arret d'une interface
# Cette fonction prend en parametre le nom de l'interface
sub ifdown
{
    my $iface = shift;

    eval { qx!$cmd_ifconfig $iface down !; };
    return (1, "Error during desactivation of $iface") if ($?);

    return (0, '');
}

# Fonction de demarrage d'une interface
# Cette fonction prend en parametre le nom de l'interface
sub ifup
{
    my $iface = shift;

    eval { qx!$cmd_ifconfig $iface up !; };
    return (1, "Error during activation of $iface") if ($?);

    return (0, '');
}

# Fonction de redemarrage de tout le reseau
# Cette fonction ne prend rien en parametre
sub network_restart
{
    eval { qx!/etc/init.d/networking restart!; };
    return (1, "Error during the restart of all interfaces") if ($?);

    return (0, '');
}

# Fonction de suppression d'une interface dans le fichier
# de configuration.
# Cette fonction prend en parametre le nom de l'interface.
sub delete_iface
{
    my ($iface) = @_;
    my $hash_conf_file;
    
    if (($hash_conf_file = _read_conf_file()) == 1)
    {
	return (1, "Error in opening /etc/network/interfaces");
    }

    # Je recupere la liste de mes interfaces au demarrage
    my @tab_auto = ($hash_conf_file->{'auto'} =~ m/\s*(\S+)\s*/g);

    # Gestion pour la ligne auto..
    my $tmp_auto;
    foreach (@tab_auto)
    {
	$tmp_auto .= $_." " if ($_ ne $iface);
    }
    chop $tmp_auto;
    $hash_conf_file->{'auto'} = $tmp_auto;

    delete($hash_conf_file->{'inter'}->{$iface}) if (exists($hash_conf_file->{'inter'}->{$iface}));

    my ($ret, $msg) = _write_conf_file($hash_conf_file);
    return (1, $msg) if ($ret == 1);
    
    return (0, "$iface: Configuration written in /etc/network/interfaces");
}

# Fonction de recuperation du status d'une interface (up ou down)
# Cette fonction prend en parametre le nom de l'interface
# et retourne 0 si l'interface est down et 1 si elle est up
sub get_if_status
{
    my ($iface) = @_;

    eval { qx!$cmd_ifconfig $iface | grep UP!; };

    return (0) if ($?);
    return (1);
}

################### PRIVATES FUNCTIONS #####################

sub _do_interfaces_changes
{
    my ($iface, $iface_hash) = @_;
    my $hash_conf_file;
    my @tab_keys;
    my $info = $iface_hash->{'on_boot'};

    # On stack les clefs dans un tableau ...
    foreach my $k (keys %{$info})
    {
	push @tab_keys, $k;
    }
    # ... pour tester s'il y a des infos (idee de mickael :)
    return (1, "No boot data found") if (!scalar(@tab_keys));

    my ($err, @errno_var) = check_ip($info) unless ($info->{'mode'} eq "dhcp");
    
    if ($err)
    {
	my $field = join(', ', @errno_var);
	return(1, "$field is/are not valid for $iface in boot configuration");
    }
    
    if (($hash_conf_file = _read_conf_file()) == 1)
    {
	return (1, "Error in opening /etc/network/interfaces");
    }

    # Par defaut on considere que l'interface n'est pas au boot
    if ($hash_conf_file->{'auto'} =~ /(.*)\s*$iface\s*$/)
    {
	$hash_conf_file->{'auto'} = $1;
    }
    elsif ($hash_conf_file->{'auto'} =~ /(.*)\s*$iface\s+(.*)$/)
    {
	$hash_conf_file->{'auto'} = $1." ".$2;
    }

    # Si elle doit etre au boot :
    if (exists $iface_hash->{'active_on_boot'})
    {
	if ($iface_hash->{'active_on_boot'} eq "yes")
	{
	    if ($hash_conf_file->{'auto'} !~ /$iface[\s\$]/)
	    {
		$hash_conf_file->{'auto'} .= " ".$iface;
	    }
	}
	elsif ($iface_hash->{'active_on_boot'} ne "no")
	{
	    return (1, "\"boot\" parameter is set badly");
	}
    }
    
    if (exists $info->{'mode'} && $info->{'mode'} eq "dhcp")
    {
	delete $hash_conf_file->{'inter'}{$iface} if (exists $hash_conf_file->{'inter'}{$iface});
	$hash_conf_file->{'inter'}{$iface}{'mode'} = "dhcp";
    }
    else
    {
	$hash_conf_file->{'inter'}{$iface}{'mode'}	= $info->{'mode'} if exists $info->{'mode'};
	$hash_conf_file->{'inter'}{$iface}{'address'}	= $info->{'ip'} if exists $info->{'ip'};
	$hash_conf_file->{'inter'}{$iface}{'netmask'}	= $info->{'mask'} if exists $info->{'mask'};
	$hash_conf_file->{'inter'}{$iface}{'broadcast'}	= $info->{'bcast'} if exists $info->{'bcast'};
	$hash_conf_file->{'inter'}{$iface}{'gateway'}	= $info->{'gw'} if exists $info->{'gw'};
    }

    my ($ret, $msg) = _write_conf_file($hash_conf_file);
    
    return (1, $msg) if ($ret == 1);
    
    return (0, "$iface: Configuration written in /etc/network/interfaces");
}

sub _read_conf_file
{
    my $info;
    my $name;

    if ( -f $config_file_path && -r _ ) {
	open(FD_READ, "< $config_file_path") or die "Error: $!.\n";
	my @tab = <FD_READ>;
	close(FD_READ);
	
	foreach my $line (@tab){
	    if ( $line =~ /^(\#.*)/) {
		$info->{'commentaires'} .= $1."\n";
		next;
	    }
	    if ($line =~ /^auto\s+(.*[\w\d:])\s*$/){
		$info->{'auto'} = $1;
		next;
	    }
	    if ($line =~ /^iface\s+([\w+\d:]+)\s+inet\d?\s+(\w+)/){
		$name = $1;
		$info->{'inter'}{$name}{'mode'} = $2;
	    }
	    else {
		if ( $line =~ /^\s*(\w+)\s+(.*)/){
		    my $label = $1;
		    my $value = $2;
		    if ( exists $param->{$label} and ($value =~ /$param->{$label}/) )
		    {
			$info->{'inter'}{$name}{$label} = $value;
		    }
		}
		elsif ( $line =~ /^\s*(\#.*)/) {
		    $info->{'inter'}{$name}{'commentaires'} = $1;
		}
	    }
	}
	return $info;
    }
    else
    {
	return(1);
    }
}

sub	_write_conf_file
{
    my $hash_conf = shift;

    open(FD, "> $config_file_path") or return(1,"Error: $!.\n");    
    
    print FD $hash_conf->{'commentaires'}."\n" if ( exists $hash_conf->{'commentaires'} );
  
    print FD "auto ".$hash_conf->{'auto'}."\n\n";
    
    if (exists $hash_conf->{'inter'}{'lo'} and $hash_conf->{'inter'}{'lo'}{'mode'}) {
	print FD "iface lo inet ".$hash_conf->{'inter'}{'lo'}{'mode'}."\n\n";
	delete $hash_conf->{'inter'}{'lo'};
    }

    foreach my $key (sort  keys %{$hash_conf->{'inter'}}) {
	print FD "iface $key inet ".$hash_conf->{'inter'}{$key}{'mode'}."\n";
	print FD "\taddress ".$hash_conf->{'inter'}{$key}{'address'}."\n"	if ( $hash_conf->{'inter'}{$key}{'address'} );
	print FD "\tbroadcast ".$hash_conf->{'inter'}{$key}{'broadcast'}."\n"	if ( $hash_conf->{'inter'}{$key}{'broadcast'} );
	print FD "\tnetmask ".$hash_conf->{'inter'}{$key}{'netmask'}."\n"	if ( $hash_conf->{'inter'}{$key}{'netmask'} );
	print FD "\tnetwork ".$hash_conf->{'inter'}{$key}{'network'}."\n"	if ( $hash_conf->{'inter'}{$key}{'network'} );
	print FD "\tgateway ".$hash_conf->{'inter'}{$key}{'gateway'}."\n"	if ( $hash_conf->{'inter'}{$key}{'gateway'} );
	print FD "\t".$hash_conf->{'inter'}{$key}{'commentaires'}."\n"	if ( $hash_conf->{'inter'}{$key}{'commentaires'} );
	print FD "\n";
    }
    close(FD);
    
    return (0, "ok");
}


# sub	_get_all_name 
# {
#     my $all;
    
#     my @infos = qx!$cmd_ifconfig -a!;
#     my $boot_ifc = _get_booted_ifaces();

#     foreach my $var (@infos) {
# 	if ( $var =~ /^([\w:]+)\s+/) {
# 	    $all->{$1}= undef ;
# 	}
#     }
    
#     foreach my $key (keys %{$boot_ifc})
#     {
# 	$all->{$key}= undef ;
#     }
    
#     return $all;
# }

# sub	_get_ifconf_all
# {
#     my $interfaces = shift;
#     my $iface;
#     my $mask;
#     my $flg = 0;
#     my @infos = qx!$cmd_ifconfig -a!;

#     foreach my $var (@infos) {
# 	if ( $var =~ /^([\w:]+)\s+/) {
# 	    $iface = $1;
# 	    $interfaces->{$1} = {} if ( ! exists $interfaces->{$1});
# 	}
# 	elsif ( $var =~ /^\s+inet ad\w+:([\d\.]*)\s+Bcast:([\d\.]*)\s+Mas\w+:([\d\.]*)\s*/) {
# 	    $interfaces->{$iface}{'inact'}{'ip'} = $1;	
# 	    $interfaces->{$iface}{'inact'}{'bcast'} = $2;
# 	    $interfaces->{$iface}{'inact'}{'mask'} = $3;
# 	}
#     }
#     return $interfaces;
# }

# Cette fontion rcupre l'actuelle configuration de l'interface
# Elle prend en paramtre la hash  remplir
sub	_get_ifconf
{
    my ($hash, $interface) = @_;
    my $iface;
    my @infos;

    if (!defined($interface))
    {
	@infos = qx!$cmd_ifconfig -a!;
    }
    else
    {
	@infos = qx!$cmd_ifconfig $interface!;
    }

    foreach my $var (@infos) {
	if ($var =~ /^([\w:]+)\s+/) {
	    $iface = $1;
	    $hash->{$1} = {} if ( ! exists $hash->{$1});
	    if ($var =~ /\s+HWaddr\s+([\w\d:]+)/) 
	    {
		$hash->{$iface}{'mac_addr'} = $1; 
	    }
	}
	elsif ( $var =~ /^\s+inet ad\w+:([\d\.]*)\s+Bcast:([\d\.]*)\s+Mas\w+:([\d\.]*)\s*/) {
	    $hash->{$iface}{'ifconfig'}{'ip'} = $1;	
	    $hash->{$iface}{'ifconfig'}{'bcast'} = $2 ;	
	    $hash->{$iface}{'ifconfig'}{'mask'} = $3 ;
	}
	elsif ( $var =~ /^\s+inet ad\w+:([\d\.]*)\s+Mas\w+:([\d\.]*)\s*/) {
	    $hash->{$iface}{'ifconfig'}{'ip'} = $1;	
	    $hash->{$iface}{'ifconfig'}{'mask'} = $2 ;	
	}
	elsif ($var =~ /^\s+UP.*RUNNING.*MTU/)
	{
	    $hash->{$iface}->{'ifconfig'}->{'slave'} = 0;
	    $hash->{$iface}->{'ifconfig'}->{'slave'} = 1 if ($var =~ /^.*SLAVE.*/);
	}
    }

    return (0);
}

# # Fonction de recuperation de la configuration d'une interface.
# sub	_get_one_ifconf
# {
#     my $interfaces = shift;
#     my $iface;
#     my @infos = qx!$cmd_ifconfig -a!;

#     foreach my $var (@infos) {
# 	if ($var =~ /^([\w:]+)\s+/) {
# 	    $iface = $1;
# 	    $interfaces->{$1} = {} if ( ! exists $interfaces->{$1});
# 	    if ($var =~ /\s+HWaddr\s+([\w\d:]+)/) 
# 	    {
# 		$interfaces->{$iface}{'mac_addr'} = $1; 
# 	    }
# 	}
# 	elsif ( $var =~ /^\s+inet ad\w+:([\d\.]*)\s+Bcast:([\d\.]*)\s+Mas\w+:([\d\.]*)\s*/) {
# 	    $interfaces->{$iface}{'ifconfig'}{'ip'} = $1;	
# 	    $interfaces->{$iface}{'ifconfig'}{'bcast'} = $2 ;	
# 	    $interfaces->{$iface}{'ifconfig'}{'mask'} = $3 ;
# 	}
# 	elsif ( $var =~ /^\s+inet ad\w+:([\d\.]*)\s+Mas\w+:([\d\.]*)\s*/) {
# 	    $interfaces->{$iface}{'ifconfig'}{'ip'} = $1;	
# 	    $interfaces->{$iface}{'ifconfig'}{'mask'} = $2 ;	
# 	}
#     }
#     return (0);
    
# }

# Recuperer la liste des interfaces et de leurs caracteristiques au boot de la machine
# Concraitement, cette fonction parse le fichier /etc/network/interfaces
# Elle prend en parametre la hash  remplir..
sub	_get_ifaces_on_boot
{
    my ($hash_return) = @_;
    my $if_name;
    my $auto;
    
    if ( -f $config_file_path && -r _ )
    {
	open (FD, $config_file_path) or die "Error: $!\n";

	while (<FD>)
	{
	    # Si la ligne est commente (ou vide), je passe  la suivante..
	    next if ($_ =~ /^\s*\#/ || $_ =~ /^\n$/);

	    if ($_ =~ /^auto(.*)$/)
	    {
	        $auto .= " ".$1." ";
	    }
	    elsif ( $_ =~ /^\s*iface\s+([\w\d:]*)\s+\w*\s+(\w*)/){
		$if_name = $1;
		$hash_return->{$if_name}{'on_boot'}{'mode'} = $2;
		$hash_return->{$if_name}{'active_on_boot'} = 'no';
	    }
	    elsif ( $_ =~ /\s+address\s+([\d\.]*)/ )
	    {
		$hash_return->{$if_name}{'on_boot'}{'ip'} = $1;
	    }
	    elsif ( $_ =~ /\s+netmask\s+([\d\.]*)/ )
	    {
		$hash_return->{$if_name}{'on_boot'}{'mask'} = $1 ;
	    }
	    elsif ( $_ =~ /\s+broadcast\s+([\d\.]*)/ )
	    {
		$hash_return->{$if_name}{'on_boot'}{'bcast'} = $1 ;
	    }
	    elsif ( $_ =~ /\s+gateway\s+([\d\.]*)/ )
	    {
		$hash_return->{$if_name}{'on_boot'}{'gw'} = $1 ;
	    }
	}

	foreach (split(" ", $auto))
	{
	    $hash_return->{$_}{'active_on_boot'} = 'yes';
	}
	close (FD);
    }
    else
    {
	print $not_open_file;
	return (1);
    }

    return (0);
}

# Cette fonction determine si les interfaces ont un cable branche
# Elle prend en parametre la hash a remplir
sub	_got_link
{
    my $interfaces = shift;
    my @infos;
    
    foreach my $iface (sort keys %{$interfaces})
    {
	@infos = qx!$cmd_ethtool $iface 2>/dev/null !;
	$interfaces->{$iface}->{"link"} = "no";
	foreach my $line (@infos)
	{
	    if ($line =~ /\s+detected:\s+(\w+)/)
	    {
		$interfaces->{$iface}->{"link"} = $1;
	    } 
	}
    }
    return (0);
}

# Test sur le path et l'existence de la commande ifconfig
sub	_test_cmd
{
    my $cmd	= shift;
    my $name	= shift;
    my $func;
    
    chomp($func = `which $cmd`);
    
    if ( ! -e $func){
	$func = $name;
    }

    if ( ! -x $func){
	print STDERR "The command $cmd is not executable.\n";
	exit 1;
    }
    return $func;
}

1;
